"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
import logging

from cfnlint.helpers import REGION_PRIMARY
from cfnlint.jsonschema import CfnTemplateValidator
from cfnlint.rules.jsonschema.base import BaseJsonSchema
from cfnlint.schema.manager import PROVIDER_SCHEMA_MANAGER, ResourceNotFoundError

LOGGER = logging.getLogger("cfnlint.rules.resources.properties.JsonSchema")


class JsonSchema(BaseJsonSchema):
    """Check Base Resource Configuration"""

    id = "E3000"
    shortdesc = "Parent rule for doing JSON schema validation"
    description = "Making sure that resources properties comply with their JSON schema"
    source_url = "https://github.com/aws-cloudformation/cfn-python-lint/blob/main/docs/cfn-resource-specification.md#properties"
    tags = ["resources"]

    def __init__(self):
        """Init"""
        super().__init__()
        self.rule_set = {
            "additionalProperties": "E3002",
            "properties": "E3002",
            "dependentExcluded": "E3023",
            "dependentRequired": "E3024",
            "required": "E3003",
            "requiredXor": "E3014",
            "requiredOr": "E3015",
            "enum": "E3030",
            "type": "E3012",
            "minLength": "E3033",
            "maxLength": "E3033",
            "uniqueItems": "E3037",
            "maximum": "E3034",
            "minimum": "E3034",
            "exclusiveMaximum": "E3034",
            "exclusiveMinimum": "E3034",
            "maxItems": "E3032",
            "minItems": "E3032",
            "pattern": "E3031",
            "oneOf": "E2523",
            "awsType": "E3008",
            "cfnSchema": "E3017",
            "cfnRegionSchema": "E3018",
            "ref": "E1020",
            "fn_base64": "E1021",
            "fn_cidr": "E1024",
            "fn_findinmap": "E1011",
            "fn_foreach": "E1032",
            "fn_getatt": "E1010",
            "fn_getazs": "E1015",
            "fn_if": "E1028",
            "fn_importvalue": "E1016",
            "fn_join": "E1022",
            "fn_length": "E1030",
            "fn_select": "E1017",
            "fn_split": "E1018",
            "fn_sub": "E1019",
            "fn_tojsonstring": "E1031",
        }
        self.child_rules = dict.fromkeys(list(self.rule_set.values()))

    def _run_json_schema_validate(self, schema, cfn, regions, p, n):
        cfn_validator = self.setup_validator(
            validator=CfnTemplateValidator,
            schema=schema.json_schema,
            context=cfn.context.create_context_for_resource_properties(
                regions=regions, resource_name=n
            ),
        ).evolve(
            cfn=cfn,
        )
        path = ["Resources", n, "Properties"]
        return self.json_schema_validate(cfn_validator, p, path)

    def match(self, cfn):
        """Check CloudFormation Properties"""
        matches = []

        for n, values in cfn.get_resources().items():
            p = values.get("Properties", {})
            t = values.get("Type", None)
            if not isinstance(t, str):
                continue
            if t.startswith("Custom::"):
                t = "AWS::CloudFormation::CustomResource"

            cached_regions = []
            cached_schema = None
            for region in cfn.regions:
                schema = {}
                try:
                    schema = PROVIDER_SCHEMA_MANAGER.get_resource_schema(region, t)
                except ResourceNotFoundError as e:
                    LOGGER.info(e)
                    continue
                if schema:
                    if schema.json_schema:
                        if not schema.is_cached and region != REGION_PRIMARY:
                            matches.extend(
                                self._run_json_schema_validate(
                                    schema, cfn, [region], p, n
                                )
                            )
                        else:
                            if not cached_schema:
                                cached_schema = schema
                            cached_regions.append(region)

            if cached_regions:
                matches.extend(
                    self._run_json_schema_validate(
                        cached_schema, cfn, cached_regions, p, n
                    )
                )

        return matches
